@@ -66,7 +66,6 @@ https://spiceypy.readthedocs.io/en/main/documentation.html
 https://naif.jpl.nasa.gov/pub/naif/toolkit_docs/C/cspice/
 
 """
-import sys
 from .common import __version__, __author__, __url__
 from .base import SpiceBase
 from .body import Body
@@ -75,6 +74,7 @@ from .observation import Observation
 from . import gui
 from . import utils
 from . import kernel_downloader
+
 __all__ = [
     'SpiceBase',
     'Body',
@@ -86,5 +86,7 @@ __all__ = [
     'kernel_downloader',
 ]
 
-def _main():
+
+def main():
+    """:meta private:"""
     gui._main()
