@@ -1,15 +1,15 @@
 import datetime
 import math
-from functools import wraps, lru_cache, partial
+from functools import lru_cache, wraps
 from typing import (
     Any,
     Callable,
+    Concatenate,
     Iterable,
     NamedTuple,
     ParamSpec,
-    TypeVar,
     Protocol,
-    Concatenate,
+    TypeVar,
 )
 
 import matplotlib.patches
@@ -21,6 +21,7 @@ from matplotlib.axes import Axes
 from spiceypy.utils.exceptions import NotFoundError
 
 from .body import Body
+from .progress import progress_decorator
 
 T = TypeVar('T')
 S = TypeVar('S')
@@ -160,6 +161,10 @@ class BodyXY(Body):
         body.get_backplane_img('EMISSION') # Takes ~10s to execute
         body.get_backplane_img('INCIDENCE') # Executes instantly
 
+    You can optionally display a progress bar for long running processes like backplane
+    generation by `show_progress=True` when creating a `BodyXY` instance (or any other
+    instance which derives from :class:`SpiceBase`).
+
     The size of the image can be specified by using the `nx` and `ny` parameters to
     specify the number of pixels in the x and y dimensions of the image respectively.
     If `nx` and `ny` are equal (i.e. the image is square), then the parameter `sz` can
@@ -879,7 +884,7 @@ class BodyXY(Body):
         x_map = self.get_x_map(degree_interval)
         y_map = self.get_y_map(degree_interval)
         interpolator = scipy.interpolate.RectBivariateSpline(
-            np.arange(img.shape[1]), np.arange(img.shape[0]), img, kx=1, ky=1
+            np.arange(img.shape[0]), np.arange(img.shape[1]), img, kx=1, ky=1
         )
         projected = self._make_empty_map(degree_interval)
         for a, b in self._iterate_image(projected.shape):
@@ -1206,10 +1211,14 @@ class BodyXY(Body):
     def _test_if_img_size_valid(self) -> bool:
         return (self._nx > 0) and (self._ny > 0)
 
-    def _iterate_image(self, shape: tuple[int, ...]) -> Iterable[tuple[int, int]]:
+    def _iterate_image(
+        self, shape: tuple[int, ...], progress: bool = False
+    ) -> Iterable[tuple[int, int]]:
         ny = shape[0]
         nx = shape[1]
         for y in range(ny):
+            if progress:
+                self._update_progress_hook(y / ny)
             for x in range(nx):
                 yield y, x
 
@@ -1252,6 +1261,7 @@ class BodyXY(Body):
         return r
 
     @_cache_clearable_result
+    @progress_decorator
     def _get_targvec_img(self) -> np.ndarray:
         out = self._make_empty_img(3)
 
@@ -1261,7 +1271,7 @@ class BodyXY(Body):
         x0 = self.get_x0()
         y0 = self.get_y0()
 
-        for y, x in self._iterate_image(out.shape):
+        for y, x in self._iterate_image(out.shape, progress=True):
             if (
                 self._optimize_speed
                 and ((x - x0) * (x - x0) + (y - y0) * (y - y0)) > r2
@@ -1283,17 +1293,21 @@ class BodyXY(Body):
         return out
 
     @_cache_stable_result
+    @progress_decorator
     def _get_targvec_map(self, degree_interval: float) -> np.ndarray:
         out = self._make_empty_map(degree_interval, 3)
         lons, lats = self._make_map_lonlat_arrays(degree_interval)
         for a, lat in enumerate(lats):
+            self._update_progress_hook(a / len(lats))
             for b, lon in enumerate(lons):
                 out[a, b] = self.lonlat2targvec(lon, lat)
         return out
 
-    def _enumerate_targvec_img(self) -> Iterable[tuple[int, int, np.ndarray]]:
+    def _enumerate_targvec_img(
+        self, progress: bool = False
+    ) -> Iterable[tuple[int, int, np.ndarray]]:
         targvec_img = self._get_targvec_img()
-        for y, x in self._iterate_image(targvec_img.shape):
+        for y, x in self._iterate_image(targvec_img.shape, progress=progress):
             targvec = targvec_img[y, x]
             if math.isnan(targvec[0]):
                 # only check if first element nan for efficiency
@@ -1301,23 +1315,27 @@ class BodyXY(Body):
             yield y, x, targvec
 
     def _enumerate_targvec_map(
-        self, degree_interval: float
+        self, degree_interval: float, progress: bool = False
     ) -> Iterable[tuple[int, int, np.ndarray]]:
         targvec_map = self._get_targvec_map(degree_interval)
-        for a, b in self._iterate_image(targvec_map.shape):
+        for a, b in self._iterate_image(targvec_map.shape, progress=progress):
             yield a, b, targvec_map[a, b]
 
     @_cache_clearable_result
+    @progress_decorator
     def _get_lonlat_img(self) -> np.ndarray:
         out = self._make_empty_img(2)
-        for y, x, targvec in self._enumerate_targvec_img():
+        for y, x, targvec in self._enumerate_targvec_img(progress=True):
             out[y, x] = self._targvec2lonlat_radians(targvec)
         return np.rad2deg(out)
 
     @_cache_stable_result
+    @progress_decorator
     def _get_lonlat_map(self, degree_interval: float) -> np.ndarray:
         out = self._make_empty_map(degree_interval, 2)
-        for a, b, targvec in self._enumerate_targvec_map(degree_interval):
+        for a, b, targvec in self._enumerate_targvec_map(
+            degree_interval, progress=True
+        ):
             out[a, b] = self._targvec2lonlat_radians(targvec)
         return np.rad2deg(out)
 
@@ -1366,17 +1384,21 @@ class BodyXY(Body):
         return self._get_lonlat_map(degree_interval)[:, :, 1]
 
     @_cache_clearable_result
+    @progress_decorator
     def _get_radec_img(self) -> np.ndarray:
         out = self._make_empty_img(2)
-        for y, x in self._iterate_image(out.shape):
+        for y, x in self._iterate_image(out.shape, progress=True):
             out[y, x] = self._xy2radec_radians(x, y)
         return np.rad2deg(out)
 
     @_cache_stable_result
+    @progress_decorator
     def _get_radec_map(self, degree_interval: float) -> np.ndarray:
         out = self._make_empty_map(degree_interval, 2)
         visible = self._get_illumf_map(degree_interval)[:, :, 4]
-        for a, b, targvec in self._enumerate_targvec_map(degree_interval):
+        for a, b, targvec in self._enumerate_targvec_map(
+            degree_interval, progress=True
+        ):
             if visible[a, b]:
                 out[a, b] = self._obsvec2radec_radians(self._targvec2obsvec(targvec))
         return np.rad2deg(out)
@@ -1426,10 +1448,11 @@ class BodyXY(Body):
         return self._get_radec_map(degree_interval)[:, :, 1]
 
     @_cache_clearable_result_with_args
+    @progress_decorator
     def _get_xy_map(self, degree_interval: float) -> np.ndarray:
         out = self._make_empty_map(degree_interval, 2)
         radec_map = self._get_radec_map(degree_interval)
-        for a, b in self._iterate_image(out.shape):
+        for a, b in self._iterate_image(out.shape, progress=True):
             ra, dec = radec_map[a, b]
             if not math.isnan(ra):
                 out[a, b] = self.radec2xy(ra, dec)
@@ -1488,16 +1511,21 @@ class BodyXY(Body):
         return self._get_xy_map(degree_interval)[:, :, 1]
 
     @_cache_clearable_result
+    @progress_decorator
     def _get_illumination_gie_img(self) -> np.ndarray:
         out = self._make_empty_img(3)
-        for y, x, targvec in self._enumerate_targvec_img():
+        for y, x, targvec in self._enumerate_targvec_img(progress=True):
+            self._update_progress_hook(y / self._ny)
             out[y, x] = self._illumination_angles_from_targvec_radians(targvec)
         return np.rad2deg(out)
 
     @_cache_stable_result
+    @progress_decorator
     def _get_illumf_map(self, degree_interval: float) -> np.ndarray:
         out = self._make_empty_map(degree_interval, 5)
-        for a, b, targvec in self._enumerate_targvec_map(degree_interval):
+        for a, b, targvec in self._enumerate_targvec_map(
+            degree_interval, progress=True
+        ):
             out[a, b] = self._illumf_from_targvec_radians(targvec)
         return np.rad2deg(out)
 
@@ -1571,11 +1599,12 @@ class BodyXY(Body):
         return self._get_illumf_map(degree_interval)[:, :, 2]
 
     @_cache_clearable_result
+    @progress_decorator
     def _get_state_imgs(self) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
         position_img = self._make_empty_img(3)
         velocity_img = self._make_empty_img(3)
         lt_img = self._make_empty_img()
-        for y, x, targvec in self._enumerate_targvec_img():
+        for y, x, targvec in self._enumerate_targvec_img(progress=True):
             (
                 position_img[y, x],
                 velocity_img[y, x],
@@ -1584,13 +1613,16 @@ class BodyXY(Body):
         return position_img, velocity_img, lt_img
 
     @_cache_stable_result
+    @progress_decorator
     def _get_state_maps(
         self, degree_interval: float
     ) -> tuple[np.ndarray, np.ndarray, np.ndarray]:
         position_map = self._make_empty_map(degree_interval, 3)
         velocity_map = self._make_empty_map(degree_interval, 3)
         lt_map = self._make_empty_map(degree_interval)
-        for a, b, targvec in self._enumerate_targvec_map(degree_interval):
+        for a, b, targvec in self._enumerate_targvec_map(
+            degree_interval, progress=True
+        ):
             (
                 position_map[a, b],
                 velocity_map[a, b],
@@ -1624,6 +1656,7 @@ class BodyXY(Body):
         return lt_map * self.speed_of_light()
 
     @_cache_clearable_result
+    @progress_decorator
     def get_radial_velocity_img(self) -> np.ndarray:
         """
         See also :func:`get_backplane_img`.
@@ -1635,13 +1668,14 @@ class BodyXY(Body):
         """
         out = self._make_empty_img()
         position_img, velocity_img, lt_img = self._get_state_imgs()
-        for y, x, targvec in self._enumerate_targvec_img():
+        for y, x, targvec in self._enumerate_targvec_img(progress=True):
             out[y, x] = self._radial_velocity_from_state(
                 position_img[y, x], velocity_img[y, x]
             )
         return out
 
     @_cache_stable_result
+    @progress_decorator
     def get_radial_velocity_map(self, degree_interval: float = 1) -> np.ndarray:
         """
         See also :func:`get_backplane_map`.
@@ -1655,7 +1689,9 @@ class BodyXY(Body):
         """
         out = self._make_empty_map(degree_interval)
         position_map, velocity_map, lt_map = self._get_state_maps(degree_interval)
-        for a, b, targvec in self._enumerate_targvec_map(degree_interval):
+        for a, b, targvec in self._enumerate_targvec_map(
+            degree_interval, progress=True
+        ):
             out[a, b] = self._radial_velocity_from_state(
                 position_map[a, b], velocity_map[a, b]
             )
