@@ -1,19 +1,19 @@
 import datetime
 import os
 import warnings
-import math
-from functools import wraps, lru_cache, partial
-from typing import Any, Callable, Iterable, NamedTuple, ParamSpec, TypeVar, Concatenate
+from typing import ParamSpec, TypeVar
 
-import numpy as np
-import PIL.Image
-from astropy.io import fits
 import astropy.wcs
-from astropy.utils.exceptions import AstropyWarning
-import scipy.ndimage
+import numpy as np
 import photutils.aperture
+import PIL.Image
+import scipy.ndimage
+from astropy.io import fits
+from astropy.utils.exceptions import AstropyWarning
+
 from . import common, utils
-from .body_xy import BodyXY, _cache_clearable_result, _cache_clearable_result_with_args
+from .body_xy import BodyXY, _cache_clearable_result_with_args
+from .progress import progress_decorator, SaveMapProgressHookCLI, SaveNavProgressHookCLI
 
 T = TypeVar('T')
 S = TypeVar('S')
@@ -387,7 +387,8 @@ class Observation(BodyXY):
 
     # Mapping
     @_cache_clearable_result_with_args
-    def mapped_data(self, degree_interval: float = 1) -> np.ndarray:
+    @progress_decorator
+    def get_mapped_data(self, degree_interval: float = 1) -> np.ndarray:
         """
         Projects the observed :attr:`data` onto a lon/lat grid using
         :func:`BodyXY.map_img`.
@@ -402,7 +403,8 @@ class Observation(BodyXY):
             visible have a value of NaN.
         """
         projected = []
-        for img in self.data:
+        for idx, img in enumerate(self.data):
+            self._update_progress_hook(idx / len(self.data))
             projected.append(self.map_img(img, degree_interval=degree_interval))
         return np.array(projected)
 
@@ -579,7 +581,7 @@ class Observation(BodyXY):
     ) -> str:
         """
         Automatically generates a useful filename from the target name and date of the
-        observation, e.g. `'JUPITER_2000-01-01T123456.fits.gz'`.
+        observation, e.g. `'JUPITER_2000-01-01T123456.fits'`.
 
         Args:
             extension: Optionally specify the file extension to add to the filename.
@@ -597,7 +599,10 @@ class Observation(BodyXY):
             suffix=suffix,
         )
 
-    def save_observation(self, path: str) -> None:
+    @progress_decorator
+    def save_observation(
+        self, path: str, show_progress: bool = False, print_info: bool = True
+    ) -> None:
         """
         Save a FITS file containing the observed data and generated backplanes.
 
@@ -612,27 +617,57 @@ class Observation(BodyXY):
 
         Args:
             path: Filepath of output file.
+            show_progress: Display a progress bar rather than printing progress info.
+                This does not have an effect if `show_progress=True` was set when
+                creating this `Observation`.
+            print_info: Toggle printing of progress information (defaults to `True`).
         """
-        print('Saving observation to', path)
+        if show_progress and self._get_progress_hook() is None:
+            print_info = False
+            self._set_progress_hook(SaveNavProgressHookCLI())
+        else:
+            show_progress = False
+
+        if print_info:
+            print('Saving observation to', path)
+
+        progress_max = 10 + len(self.backplanes)
         with utils.filter_fits_comment_warning():
             data = self.data
             header = self.header.copy()
 
+            self._update_progress_hook(1 / progress_max)
+
             self.add_header_metadata(header)
             hdul = fits.HDUList([fits.PrimaryHDU(data=data, header=header)])
-            for name, backplane in self.backplanes.items():
-                print(' Creating backplane ', name)
+            for bp_idx, (name, backplane) in enumerate(self.backplanes.items()):
+                self._update_progress_hook((bp_idx + 1) / progress_max)
+                if print_info:
+                    print(' Creating backplane:', name)
                 img = backplane.get_img()
                 header = fits.Header([('ABOUT', backplane.description)])
                 header.add_comment('Backplane generated by Planet Mapper software.')
                 hdu = fits.ImageHDU(data=img, header=header, name=name)
                 hdul.append(hdu)
+            if print_info:
+                print(' Saving file...')
             utils.check_path(path)
             hdul.writeto(path, overwrite=True)
-        print('File saved')
+        if print_info:
+            print('File saved')
 
+        if show_progress:
+            self._update_progress_hook(1)
+            self._remove_progress_hook()
+
+    @progress_decorator
     def save_mapped_observation(
-        self, path: str, include_backplanes: bool = True, degree_interval: float = 1
+        self,
+        path: str,
+        include_backplanes: bool = True,
+        degree_interval: float = 1,
+        show_progress: bool = False,
+        print_info: bool = True,
     ) -> None:
         """
         Save a FITS file containing the mapped observation in a cylindrical projection.
@@ -648,12 +683,29 @@ class Observation(BodyXY):
             include_backplanes: Toggle generating and saving backplanes to output FITS
                 file.
             degree_interval: Interval in degrees between the longitude/latitude points.
+            show_progress: Display a progress bar rather than printing progress info.
+                This does not have an effect if `show_progress=True` was set when
+                creating this `Observation`.
+            print_info: Toggle printing of progress information (defaults to `True`).
         """
-        print('Saving map to', path)
+        if show_progress and self._get_progress_hook() is None:
+            print_info = False
+            self._set_progress_hook(SaveMapProgressHookCLI(len(self.data)))
+        else:
+            show_progress = False
+
+        if print_info:
+            print('Saving map to', path)
+
+        progress_max = 15 + (len(self.backplanes) if include_backplanes else 0)
         with utils.filter_fits_comment_warning():
-            data = self.mapped_data(degree_interval)
+            if print_info:
+                print(' Projecting mapped data...')
+            data = self.get_mapped_data(degree_interval)
             header = self.header.copy()
 
+            self._update_progress_hook(1 / progress_max)
+
             self.add_header_metadata(header)
             self.append_to_header(
                 'DEGREE-INTERVAL',
@@ -665,8 +717,10 @@ class Observation(BodyXY):
 
             hdul = fits.HDUList([fits.PrimaryHDU(data=data, header=header)])
             if include_backplanes:
-                for name, backplane in self.backplanes.items():
-                    print(' Creating backplane ', name)
+                for bp_idx, (name, backplane) in enumerate(self.backplanes.items()):
+                    self._update_progress_hook((bp_idx + 1) / progress_max)
+                    if print_info:
+                        print(' Creating backplane:', name)
                     img = backplane.get_map(degree_interval)
                     header = fits.Header([('ABOUT', backplane.description)])
                     header.add_comment('Backplane generated by Planet Mapper software.')
@@ -674,9 +728,16 @@ class Observation(BodyXY):
 
                     hdu = fits.ImageHDU(data=img, header=header, name=name)
                     hdul.append(hdu)
+            if print_info:
+                print(' Saving file...')
             utils.check_path(path)
             hdul.writeto(path, overwrite=True)
-        print('File saved')
+        if print_info:
+            print('File saved')
+
+        if show_progress:
+            self._update_progress_hook(1)
+            self._remove_progress_hook()
 
     def _add_map_wcs_to_header(
         self, header: fits.Header, degree_interval: float
@@ -709,7 +770,9 @@ class Observation(BodyXY):
         Run an interactive GUI to display and adjust the fitted observation.
 
         This modifies the `Observation` object in-place, so can be used within a script
-        to e.g. interactively fit the planet's disc ::
+        to e.g. interactively fit the planet's disc. Simply run the GUI, adjust the
+        parameters until the disc is fit, then close the GUI and the `Observation`
+        object will have your new values: ::
 
             # Loaad in some datta
             observation = planetmapper.Observation('exciting_data.fits')
@@ -719,11 +782,11 @@ class Observation(BodyXY):
 
             # At this point, you can use the manually fitted observation
             observation.plot_wireframe_xy()
-        
+
         .. note ::
 
             The `Open...` button is disabled for user interfaces created by this method
-            to ensure that only one :class:`Observation` object is modified by thte user
+            to ensure that only one :class:`Observation` object is modified by the user
             interface.
 
             If you want the full user interface functionality instead, then call
@@ -731,6 +794,7 @@ class Observation(BodyXY):
             manually using :func:`planetmapper.gui.GUI.run`.
         """
         from .gui import GUI
+
         gui = GUI(allow_open=False)
         gui.set_observation(self)
         gui.run()
